@@ -9,11 +9,11 @@ use std::ptr;
 use std::sync::mpsc::{channel, Receiver, SendError, Sender};
 use std::thread::{self, JoinHandle};
 use windows::Win32::Foundation;
+use windows::Win32::Foundation::HANDLE;
 use windows::Win32::Foundation::WAIT_OBJECT_0;
 use windows::Win32::Media::Audio;
 use windows::Win32::System::SystemServices;
 use windows::Win32::System::Threading;
-use windows::Win32::System::WindowsProgramming;
 
 pub struct Stream {
     /// The high-priority audio processing thread calling callbacks.
@@ -150,8 +150,7 @@ impl Stream {
     fn push_command(&self, command: Command) -> Result<(), SendError<Command>> {
         self.commands.send(command)?;
         unsafe {
-            let result = Threading::SetEvent(self.pending_scheduled_event);
-            assert_ne!(result, false);
+            Threading::SetEvent(self.pending_scheduled_event).unwrap();
         }
         Ok(())
     }
@@ -160,10 +159,10 @@ impl Stream {
 impl Drop for Stream {
     #[inline]
     fn drop(&mut self) {
-        if let Ok(_) = self.push_command(Command::Terminate) {
+        if self.push_command(Command::Terminate).is_ok() {
             self.thread.take().unwrap().join().unwrap();
             unsafe {
-                Foundation::CloseHandle(self.pending_scheduled_event);
+                let _ = Foundation::CloseHandle(self.pending_scheduled_event);
             }
         }
     }
@@ -186,7 +185,7 @@ impl Drop for StreamInner {
     #[inline]
     fn drop(&mut self) {
         unsafe {
-            Foundation::CloseHandle(self.event);
+            let _ = Foundation::CloseHandle(self.event);
         }
     }
 }
@@ -238,14 +237,14 @@ fn wait_for_handle_signal(handles: &[Foundation::HANDLE]) -> Result<usize, Backe
     let result = unsafe {
         Threading::WaitForMultipleObjectsEx(
             handles,
-            false,                        // Don't wait for all, just wait for the first
-            WindowsProgramming::INFINITE, // TODO: allow setting a timeout
-            false,                        // irrelevant parameter here
+            false,               // Don't wait for all, just wait for the first
+            Threading::INFINITE, // TODO: allow setting a timeout
+            false,               // irrelevant parameter here
         )
     };
     if result == Foundation::WAIT_FAILED {
         let err = unsafe { Foundation::GetLastError() };
-        let description = format!("`WaitForMultipleObjectsEx failed: {}", err.0);
+        let description = format!("`WaitForMultipleObjectsEx failed: {:?}", err);
         let err = BackendSpecificError { description };
         return Err(err);
     }
@@ -270,6 +269,8 @@ fn run_input(
     data_callback: &mut dyn FnMut(&Data, &InputCallbackInfo),
     error_callback: &mut dyn FnMut(StreamError),
 ) {
+    boost_current_thread_priority();
+
     loop {
         match process_commands_and_await_signal(&mut run_ctxt, error_callback) {
             Some(ControlFlow::Break) => break,
@@ -297,6 +298,8 @@ fn run_output(
     data_callback: &mut dyn FnMut(&mut Data, &OutputCallbackInfo),
     error_callback: &mut dyn FnMut(StreamError),
 ) {
+    boost_current_thread_priority();
+
     loop {
         match process_commands_and_await_signal(&mut run_ctxt, error_callback) {
             Some(ControlFlow::Break) => break,
@@ -319,6 +322,17 @@ fn run_output(
     }
 }
 
+fn boost_current_thread_priority() {
+    unsafe {
+        let thread_id = Threading::GetCurrentThreadId();
+
+        let _ = Threading::SetThreadPriority(
+            HANDLE(thread_id as isize),
+            Threading::THREAD_PRIORITY_TIME_CRITICAL,
+        );
+    }
+}
+
 enum ControlFlow {
     Break,
     Continue,
